@@ -24,31 +24,24 @@ static gchar* opt_local_path;
 static gboolean opt_download;
 static gboolean opt_noprogress;
 static gboolean opt_dryrun;
+static gboolean opt_no_previews;
 static mega_session* s;
 
 static GOptionEntry entries[] =
 {
-  { "remote",        'r',   0, G_OPTION_ARG_STRING,  &opt_remote_path,  "Remote directory",                 "PATH"  },
-  { "local",         'l',   0, G_OPTION_ARG_STRING,  &opt_local_path,   "Local directory",                  "PATH"  },
-  { "download",      'd',   0, G_OPTION_ARG_NONE,    &opt_download,     "Download files from mega",         NULL    },
-  { "no-progress",   '\0',  0, G_OPTION_ARG_NONE,    &opt_noprogress,   "Disable progress bar",             NULL    },
-  { "dryrun",        'n',   0, G_OPTION_ARG_NONE,    &opt_dryrun,       "Don't perform any actual changes", NULL    },
+  { "remote",            'r',   0, G_OPTION_ARG_STRING,  &opt_remote_path,  "Remote directory",                 "PATH"  },
+  { "local",             'l',   0, G_OPTION_ARG_STRING,  &opt_local_path,   "Local directory",                  "PATH"  },
+  { "download",          'd',   0, G_OPTION_ARG_NONE,    &opt_download,     "Download files from mega",         NULL    },
+  { "no-progress",       '\0',  0, G_OPTION_ARG_NONE,    &opt_noprogress,   "Disable progress bar",             NULL    },
+  { "dryrun",            'n',   0, G_OPTION_ARG_NONE,    &opt_dryrun,       "Don't perform any actual changes", NULL    },
+  { "disable-previews",  '\0',  0, G_OPTION_ARG_NONE,    &opt_no_previews,  "Don't generate previews",          NULL    },
   { NULL }
 };
 
 static gboolean status_callback(mega_status_data* data, gpointer userdata)
 {
   if (!opt_noprogress && data->type == MEGA_STATUS_PROGRESS)
-  {
-    gchar* done_str = g_format_size_full(data->progress.done, G_FORMAT_SIZE_IEC_UNITS);
-    gchar* total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
-
-    if (data->progress.total > 0)
-      g_print("  " ESC_GREEN "%" G_GUINT64_FORMAT "%%" ESC_NORMAL " - " ESC_GREEN "%s" ESC_NORMAL " of %s" ESC_CLREOL "\r", 100 * data->progress.done / data->progress.total, done_str, total_str);
-
-    g_free(done_str);
-    g_free(total_str);
-  }
+    tool_show_progress("copying...", data);
 
   return FALSE;
 }
@@ -126,24 +119,27 @@ static gboolean up_sync_dir(GFile* root, GFile* file, const gchar* remote_path)
     return FALSE;
   }
 
+  gboolean status = TRUE;
   while ((i = g_file_enumerator_next_file(e, NULL, NULL)))
   {
     const gchar* name = g_file_info_get_name(i);
     GFile* child = g_file_get_child(file, name);
-    GFileType type = g_file_query_file_type(child, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL);
+    GFileType type = g_file_query_file_type(child, 0, NULL);
     gchar* child_remote_path = g_strconcat(remote_path, "/", name, NULL);
 
     if (type == G_FILE_TYPE_DIRECTORY)
     {
-      up_sync_dir(root, child, child_remote_path);
+      if (!up_sync_dir(root, child, child_remote_path))
+        status = FALSE;
     }
     else if (type == G_FILE_TYPE_REGULAR)
     {
-      up_sync_file(root, child, child_remote_path);
+      if (!up_sync_file(root, child, child_remote_path))
+        status = FALSE;
     }
     else
     {
-      g_printerr("ERROR: Skipping file %s\n", g_file_get_relative_path(root, file));
+      g_printerr("WARNING: Skipping special file %s\n", g_file_get_relative_path(root, file));
     }
 
     g_free(child_remote_path);
@@ -152,7 +148,7 @@ static gboolean up_sync_dir(GFile* root, GFile* file, const gchar* remote_path)
   }
 
   g_object_unref(e);
-  return TRUE;
+  return status;
 }
 
 // download operation
@@ -210,7 +206,7 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
   }
   else
   {
-    if (g_file_query_file_type(file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) != G_FILE_TYPE_DIRECTORY)
+    if (g_file_query_file_type(file, 0, NULL) != G_FILE_TYPE_DIRECTORY)
     {
       g_printerr("ERROR: Can't create local directory %s: file exists\n", local_path);
       return FALSE;
@@ -218,6 +214,7 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
   }
 
   // sync children
+  gboolean status = TRUE;
   GSList* children = mega_session_get_node_chilren(s, node), *i;
   for (i = children; i; i = i->next)
   {
@@ -227,11 +224,13 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
 
     if (child->type == MEGA_NODE_FILE)
     {
-      dl_sync_file(child, child_file, child_remote_path);
+      if (!dl_sync_file(child, child_file, child_remote_path))
+        status = FALSE;
     }
     else
     {
-      dl_sync_dir(child, child_file, child_remote_path);
+      if (!dl_sync_dir(child, child_file, child_remote_path))
+        status = FALSE;
     }
 
     g_object_unref(child_file);
@@ -239,14 +238,14 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
   }
 
   g_slist_free(children);
-  return TRUE;
+  return status;
 }
 
 // main program
 
 int main(int ac, char* av[])
 {
-  tool_init(&ac, &av, "- synchronize local and remote mega.nz directories", entries);
+  tool_init(&ac, &av, "- synchronize local and remote mega.nz directories", entries, TOOL_INIT_AUTH);
 
   if (!opt_local_path || !opt_remote_path)
   {
@@ -254,13 +253,15 @@ int main(int ac, char* av[])
     return 1;
   }
 
-  s = tool_start_session();
+  s = tool_start_session(TOOL_SESSION_OPEN);
   if (!s)
   {
     tool_fini(NULL);
     return 1;
   }
 
+  mega_session_enable_previews(s, !opt_no_previews);
+
   mega_session_watch_status(s, status_callback, NULL);
 
   // check remote dir existence
@@ -278,10 +279,12 @@ int main(int ac, char* av[])
 
   // check local dir existence
   GFile* local_file = g_file_new_for_path(opt_local_path);
+  gint status = 0;
 
   if (opt_download)
   {
-    dl_sync_dir(remote_dir, local_file, opt_remote_path);
+    if (!dl_sync_dir(remote_dir, local_file, opt_remote_path))
+      status = 1;
   }
   else
   {
@@ -291,13 +294,15 @@ int main(int ac, char* av[])
       goto err1;
     }
 
-    up_sync_dir(local_file, local_file, opt_remote_path);
+    if (!up_sync_dir(local_file, local_file, opt_remote_path))
+      status = 1;
+
     mega_session_save(s, NULL);
   }
 
   g_object_unref(local_file);
   tool_fini(s);
-  return 0;
+  return status;
 
 err1:
   g_object_unref(local_file);
