@@ -30,9 +30,9 @@ static gboolean opt_print_names = FALSE;
 
 static GOptionEntry entries[] =
 {
-  { "path",          '\0',   0, G_OPTION_ARG_FILENAME,  &opt_path,  "Local directory or file name, to save data to",  "PATH" },
-  { "no-progress",   '\0',   0, G_OPTION_ARG_NONE,    &opt_noprogress,  "Disable progress bar",   NULL},
-  { "print-names",   '\0',   0, G_OPTION_ARG_NONE,    &opt_print_names,  "Print names of downloaded files",   NULL},
+  { "path",          '\0',   0, G_OPTION_ARG_FILENAME,  &opt_path,        "Local directory or file name, to save data to",  "PATH" },
+  { "no-progress",   '\0',   0, G_OPTION_ARG_NONE,      &opt_noprogress,  "Disable progress bar",                           NULL  },
+  { "print-names",   '\0',   0, G_OPTION_ARG_NONE,      &opt_print_names, "Print names of downloaded files",                NULL  },
   { NULL }
 };
 
@@ -53,13 +53,7 @@ static gboolean status_callback(mega_status_data* data, gpointer userdata)
   }
 
   if (!opt_noprogress && data->type == MEGA_STATUS_PROGRESS)
-  {
-    gc_free gchar* done_str = g_format_size_full(data->progress.done, G_FORMAT_SIZE_IEC_UNITS);
-    gc_free gchar* total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
-
-    if (data->progress.total > 0)
-      g_print(ESC_WHITE "%s" ESC_NORMAL ": " ESC_GREEN "%" G_GUINT64_FORMAT "%%" ESC_NORMAL " - " ESC_GREEN "%s" ESC_NORMAL " of %s" ESC_CLREOL "\r", cur_file, 100 * data->progress.done / data->progress.total, done_str, total_str);
-  }
+    tool_show_progress(cur_file, data);
 
   return FALSE;
 }
@@ -124,6 +118,7 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
 
   // sync children
   GSList* children = mega_session_get_node_chilren(s, node), *i;
+  gboolean status = TRUE;
   for (i = children; i; i = i->next)
   {
     mega_node* child = i->data;
@@ -132,16 +127,18 @@ static gboolean dl_sync_dir(mega_node* node, GFile* file, const gchar* remote_pa
 
     if (child->type == 0)
     {
-      dl_sync_file(child, child_file, child_remote_path);
+      if (!dl_sync_file(child, child_file, child_remote_path))
+        status = FALSE;
     }
     else
     {
-      dl_sync_dir(child, child_file, child_remote_path);
+      if (!dl_sync_dir(child, child_file, child_remote_path))
+        status = FALSE;
     }
   }
 
   g_slist_free(children);
-  return TRUE;
+  return status;
 }
 
 int main(int ac, char* av[])
@@ -151,7 +148,7 @@ int main(int ac, char* av[])
   gint i;
   int status = 0;
 
-  tool_init_bare(&ac, &av, "- download exported files from mega.nz", entries);
+  tool_init(&ac, &av, "- download exported files from mega.nz", entries, 0);
 
   if (!strcmp(opt_path, "-"))
   {
@@ -187,7 +184,7 @@ int main(int ac, char* av[])
 
   // create session
 
-  s = mega_session_new();
+  s = tool_start_session(0);
 
   mega_session_watch_status(s, status_callback, NULL);
 
@@ -212,7 +209,7 @@ int main(int ac, char* av[])
           g_print("\r" ESC_CLREOL "\n");
         g_printerr("ERROR: Download failed for '%s': %s\n", link, local_err->message);
         g_clear_error(&local_err);
-        status++;
+        status = 1;
       }
       else
       {
@@ -239,6 +236,7 @@ int main(int ac, char* av[])
       {
         g_printerr("ERROR: Can't open folder '%s': %s\n", link, local_err->message);
         g_clear_error(&local_err);
+        status = 1;
       }
       else
       {
@@ -253,16 +251,19 @@ int main(int ac, char* av[])
           if (g_file_query_file_type(local_dir, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL) == G_FILE_TYPE_DIRECTORY)
           {
             gc_free gchar* node_path = mega_node_get_path_dup(root_node);
-            dl_sync_dir(root_node, local_dir, node_path);
+            if (!dl_sync_dir(root_node, local_dir, node_path))
+              status = 1;
           }
           else
           {
             g_printerr("ERROR: %s must be a directory\n", opt_path);
+            status = 1;
           }
         }
         else
         {
           g_printerr("ERROR: EXP folder fs has multiple toplevel nodes? Weird!\n");
+          status = 1;
         }
 
         g_slist_free(l);
