@@ -21,11 +21,13 @@
 
 static gchar* opt_path = "/Root";
 static gboolean opt_noprogress = FALSE;
+static gboolean opt_no_previews;
 
 static GOptionEntry entries[] =
 {
-  { "path",          '\0',   0, G_OPTION_ARG_STRING,  &opt_path,  "Remote path to save files to",  "PATH" },
-  { "no-progress",   '\0',   0, G_OPTION_ARG_NONE,    &opt_noprogress,  "Disable progress bar",   NULL},
+  { "path",             '\0',   0, G_OPTION_ARG_STRING,  &opt_path,         "Remote path to save files to",          "PATH" },
+  { "no-progress",      '\0',   0, G_OPTION_ARG_NONE,    &opt_noprogress,   "Disable progress bar",                  NULL   },
+  { "disable-previews", '\0',   0, G_OPTION_ARG_NONE,    &opt_no_previews,  "Don't generate previews",               NULL   },
   { NULL }
 };
 
@@ -35,17 +37,12 @@ static gboolean status_callback(mega_status_data* data, gpointer userdata)
 {
   if (data->type == MEGA_STATUS_FILEINFO)
   {
+    g_free(cur_file);
     cur_file = g_strdup(data->fileinfo.name);
   }
 
   if (!opt_noprogress && data->type == MEGA_STATUS_PROGRESS)
-  {
-    gc_free gchar* done_str = g_format_size_full(data->progress.done, G_FORMAT_SIZE_IEC_UNITS);
-    gc_free gchar* total_str = g_format_size_full(data->progress.total, G_FORMAT_SIZE_IEC_UNITS);
-
-    if (data->progress.total > 0)
-      g_print(ESC_WHITE "%s" ESC_NORMAL ": " ESC_GREEN "%" G_GUINT64_FORMAT "%%" ESC_NORMAL " - " ESC_GREEN "%s" ESC_NORMAL " of %s" ESC_CLREOL "\r", cur_file, 100 * data->progress.done / data->progress.total, done_str, total_str);
-  }
+    tool_show_progress(cur_file, data);
 
   return FALSE;
 }
@@ -55,7 +52,7 @@ int main(int ac, char* av[])
   gc_error_free GError *local_err = NULL;
   mega_session* s;
 
-  tool_init(&ac, &av, "- upload files to mega.nz", entries);
+  tool_init(&ac, &av, "- upload files to mega.nz", entries, TOOL_INIT_AUTH);
 
   if (ac < 2)
   {
@@ -64,16 +61,21 @@ int main(int ac, char* av[])
     return 1;
   }
 
-  s = tool_start_session();
+  s = tool_start_session(TOOL_SESSION_OPEN);
   if (!s)
     return 1;
 
+  mega_session_enable_previews(s, !opt_no_previews);
+
   mega_session_watch_status(s, status_callback, NULL);
 
+  gint status = 0;
   gint i;
   for (i = 1; i < ac; i++)
   {
     gc_free gchar* path = tool_convert_filename(av[i], TRUE);
+
+    g_free(cur_file);
     cur_file = g_path_get_basename(path);
 
     // perform download
@@ -84,6 +86,8 @@ int main(int ac, char* av[])
 
       g_printerr("ERROR: Upload failed for '%s': %s\n", path, local_err->message);
       g_clear_error(&local_err);
+
+      status = 1;
     }
     else
     {
@@ -95,5 +99,5 @@ int main(int ac, char* av[])
   mega_session_save(s, NULL);
 
   tool_fini(s);
-  return 0;
+  return status;
 }
